using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Windows.Forms;

namespace SmartLauncher
{
	static class RepositoryEngine
	{
		static SortedList<string, RepositoryItem> files;
		static object lockObject = new object();

		#region "Constructors"
		static RepositoryEngine()
		{
			files = new SortedList<string, RepositoryItem>();
		}
		#endregion

		#region "Procedures"
		public static bool Scan(bool useLocal, bool useFilter)
		{
			string scannedDirectory;

			lock (lockObject)
				files.Clear();

			if (useLocal == true)
				scannedDirectory = ApplicationStartInfo.LocalRoot;
			else
				scannedDirectory = Properties.Settings.Default.RepositoryHome;

			if (Directory.Exists(scannedDirectory) == false)
			{
				if (Properties.Settings.Default.SendBugReport == false)
					System.Windows.Forms.MessageBox.Show(string.Format("The {0} repository cannot be found.\n\nPlease, contact your IT helpdesk.", (useLocal == true ? "local" : "remote")), Properties.Settings.Default.ShellHome, MessageBoxButtons.OK, MessageBoxIcon.Stop);
				else
					throw new System.IO.DirectoryNotFoundException(string.Format("The {0} repository cannot be found.\n\nPlease, contact your IT helpdesk.", (useLocal == true ? "local" : "remote")));


				lock (lockObject)
					files.Clear();
				return false;
			}
			else
				Scan(scannedDirectory, useLocal, useFilter);

			// Enlever tous les fichiers  la racine du repository

			lock (lockObject)
				for (int idx = 0; idx < files.Count; )
				{
					RepositoryItem item = files.Values[idx];
					if ((item.FolderName == null) || (item.FolderName.Length == 0))
						files.RemoveAt(idx);
					else
						idx++;
				}

			return true;
		}

		static void Scan(string rootDir, bool useLocal, bool useFilter)
		{
			string[] items;
			string validExt = Properties.Settings.Default.IncludeExt.ToLower();

			items = Directory.GetFiles(rootDir);
			foreach (string item in items)
			{
				if ((useFilter == false) || ((validExt.Contains(Path.GetExtension(item).ToLower()))) && (IsExcluded(item) == false))
				{

					RepositoryItem lf = new RepositoryItem(item, useLocal);
					lock (lockObject)
						if (files.ContainsKey(lf.Key) == false)
							files.Add(lf.Key, lf);
				}
			}

			items = Directory.GetDirectories(rootDir);
			foreach (string item in items)
				Scan(item, useLocal, useFilter);
		}

		static bool IsExcluded(string filePath)
		{
			string[] items = Properties.Settings.Default.ExcludeExt.ToLower().Split(' ');
			string file = filePath.ToLower();

			foreach (string item in items)
				if (file.Contains(item) == true)
					return true;

			return false;
		}
		#endregion

		#region "Properties"
		public static SortedList<string, RepositoryItem> Files
		{
			get
			{
				SortedList<string, RepositoryItem> rc;

				rc = new SortedList<string, RepositoryItem>();

				lock (lockObject)
					foreach (KeyValuePair<string, RepositoryItem> item in files)
						rc.Add(item.Key, item.Value);

				return rc;
				// return files;
			}
		}
		#endregion
	}
}
