using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;

namespace SmartLauncher
{
	class RepositoryItem
	{
		// Basic Information
		string absolutePath;    // C:\Repository\Folder\File.ext
		string repositoryPath;  // C:\Repository
		string relativePath;    //               Folder\File.ext
		string folderName;      //               Folder
		string fileName;        //                      File.ext
		long fileSize;

		// Extra information
		int imageIndex;
		string productName;
		string productVersion;
		DateTime versionDate;

		#region "Constructors"
		public RepositoryItem(string filePath, bool isLocal)
		{
			absolutePath = filePath;
			if (isLocal == true)
				repositoryPath = ApplicationStartInfo.LocalRoot;
			else
				repositoryPath = Properties.Settings.Default.RepositoryHome;
			relativePath = absolutePath.Substring(repositoryPath.Length);
			folderName = Path.GetDirectoryName(absolutePath).Substring(repositoryPath.Length);
			fileName = Path.GetFileName(absolutePath);

			if ((isLocal == true) && (File.Exists(filePath) == true))
			{
				// Rechercher les proprits tendues du fichier
				imageIndex = IconEngine.AddFileIcon(filePath);
				FileVersionInfo fvi = FileVersionInfo.GetVersionInfo(filePath);
				if ((fvi.ProductName == null) || (fvi.ProductName.Length == 0))
					productName = Path.GetFileNameWithoutExtension(fileName);
				else
					productName = Path.GetFileNameWithoutExtension(fvi.ProductName);
				productVersion = fvi.ProductVersion;
			}

			FileInfo fi = new FileInfo(filePath);
			try
			{
				fileSize = fi.Length;
				versionDate = fi.LastWriteTime;
			}
			catch
			{
			}
		}
		#endregion

		#region "Properties"
		public string Key
		{
			get { return relativePath.ToLower(); }
		}

		public string AbsolutePath
		{
			get { return absolutePath; }
		}

		public string RepositoryPath
		{
			get { return repositoryPath; }
			set
			{
				repositoryPath = value;
				absolutePath = repositoryPath + relativePath;
			}
		}

		public string RelativePath
		{
			get { return relativePath; }
		}

		public string FolderName
		{
			get { return folderName; }
		}

		public string FileName
		{
			get { return fileName; }
		}

		public int ImageIndex
		{
			get { return imageIndex; }
		}

		public string ProductName
		{
			get { return productName; }
		}

		public string ProductVersion
		{
			get { return productVersion; }
		}

		public DateTime VersionDate
		{
			get { return versionDate; }
		}

		public long Size
		{
			get { return fileSize; }
		}
		#endregion
	}
}
